==> spacejunk-1.0.3.tar.gz: spacejunk-1.0.3/src/resourcemanager.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourcemanager.h"
#include <SDL_image.h>
#include <algorithm>
#include <functional>
#include "parsercombinators.h"
#include <fstream>
#include <iostream>
#include <SFont.h>
#include <unistd.h>
#include <global.h>
#include "debugmsg.h"
#include "sge.h"

using namespace std;

// Spanish special characters
basic_string<wchar_t> spchars;


Mix_Music * MusicLoader::create(const std::string & name) {
    string path=music_paths[name];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Music "<<name<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load sound if it is not loaded
    SDL_RWops *rw=SDL_RWFromFile(("data/"+path).c_str(),"r");
    if (!rw) {
        CHERROR<<SDL_GetError()<<ENDL;
        CHERROR<<"ResourceManager error: File "<<path<<" could not be loaded."<<ENDL;
        exit(1);
    }

    Mix_Music * s=Mix_LoadMUS_RW(rw);
    if (!s) {
        CHERROR<<SDL_GetError()<<ENDL;
        CHERROR<<"ResourceManager error: Music file "<<path<<" could not be loaded."<<ENDL;
        exit(1);
    }

    return s;
};

Parser & MusicLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    S("<musics>");
    MANYbegin;
    TRY(S("<music"))WS;
    S("name=")PATTTEXT(&name);
    WS;
    S("path=")PATTTEXT(&path)S("/>")WS;
    music_paths[name]=path;
    MANYend;
    S("</musics>");
    WS;
    PARSEend;
    return p;
};


Mix_Chunk * SoundLoader::create(const std::string & name) {
    string path=sound_paths[name];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Sound "<<name<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load sound if it is not loaded
    SDL_RWops *rw=SDL_RWFromFile(("data/"+path).c_str(),"r");
    if (!rw) {
        CHERROR<<"ResourceManager error: File "<<path<<" could not be loaded."<<SDL_GetError()<<ENDL;
        exit(1);
    }

    Mix_Chunk * s=Mix_LoadWAV_RW(rw,false);
	SDL_RWclose(rw);
    if (!s) {
        CHERROR<<"ResourceManager error: Sound file "<<ENDL<<" could not be loaded."<<ENDL;
        exit(1);
    }

    return s;
};
Parser & SoundLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    LS("<sounds>");
    MANYbegin;
    string text;
    TRY(S("<sound"))WS;
    S("name=")PATTTEXT(&name);
    WS;
    S("path=")PATTTEXT(&path)S("/>")WS;
    sound_paths[name]=path;
    MANYend;
    S("</sounds>");
    WS;
    PARSEend;
    return p;
};



Palette * PaletteLoader::create(const std::string & id) {
    string path=palette_paths[id];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Palette "<<id<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load palette if it is not loaded
    FILE * f=fopen(("data/"+path).c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open Palette file: "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer tok(f);
    Lexer p(&tok);
    Palette * pal=new Palette();
    int red,green,blue;
    PARSEbegin(Lexer,p);
    pal->ncolors=0;
	NEOLN;PEOLN
	NEOLN;PEOLN
	NEOLN;PEOLN
	NEOLN;PEOLN
    WS;
    MANYbegin;
    if (pal->ncolors==256) break;
    LINT(&red)LINT(&green)LINT(&blue);
    pal->colors[pal->ncolors].r=red;
    pal->colors[pal->ncolors].g=green;
    pal->colors[pal->ncolors].b=blue;
    pal->ncolors++;
    NEOLN;
    WS;
    MANYend;

    PARSEend;
    if (p.error()) {
        fclose(f);
        CHERROR<<"ResourceManager error: Palette file "<<("data/"+path)<<" could not be loaded: "<<p.errorMessage()<<ENDL;
        exit(1);
    }
    fclose(f);

    return pal;
};
Parser & PaletteLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    LS("<palettes>")
    MANYbegin;
    TRY(S("<palette"));
    WS;
    S("name=")PATTTEXT(&name)S("path=")PATTTEXT(&path)S("/>");
    palette_paths[name]=path;
    MANYend;
    S("</palettes>")
    PARSEend;
    return p;
};


void Custom_FreeChunk(Mix_Chunk * mc) {
    for (int i=Mix_AllocateChannels(-1);i>=0;i--)
        if (Mix_GetChunk(i)==&*mc) Mix_HaltChannel(i);
    Mix_FreeChunk(mc);
}

std::string TextLoader::create(const std::string & id) {
    std::map<std::string,std::string>::iterator i=texts.find(id);
    if (i!=texts.end()) 
		return i->second;
    else {
        CHERROR<<"ResourceManager error: Text "<<id<<" is not defined."<<ENDL;
        exit(1);
    }
};
Parser & TextLoader::parse(Parser & p,std::string * id) {
    string name;
    PARSEbegin(Parser,p);
    LS("<texts>");
    MANYbegin;
    basic_string<wchar_t> text;
    TRY(S("<text"))WS;
    S("name=")PATTTEXT(&name)S(">")WS;
    PATTTEXT(&text)
    texts[name]=wstos(text);
    S("</text>")WS;
    MANYend;
    S("</texts>");
    WS;
    PARSEend;
    return p;
};

std::vector<std::string> * LevelLoader::create(const std::string & id) {
    return &levels;
};
Parser & LevelLoader::parse(Parser & p,std::string * id) {
    string name,path;
    PARSEbegin(Parser,p);
    LS("<levels>");
    MANYbegin;
    TRY(S("<level"))WS;
    S("path=")PATTTEXT(&path)S("/>");
    levels.push_back(path);
    MANYend;
    S("</levels>");
    PARSEend;
    return p;
};



void readResources(const std::string& path,std::list<RLoader*> & rl) {
    FILE * f=fopen(path.c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open resource file: "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer ct(f);
    ConfigTokenizer t(&ct);
    Parser p=&t;

    PARSEbegin(Parser,p);
    PWS;
    S("<resources>")WS;
    S("<specialcharacterset>") PATTTEXT(&spchars)S("</specialcharacterset>")WS;
    for (unsigned int i=0;i<spchars.size();i++)
        SFont_map[spchars[i]-127]=127+i;

    list<RLoader*>::iterator i=rl.begin();
    while(i!=rl.end() && !p.error()) {
        OPTbegin;
            P1((*i)->parse,NULL);
            i++;
        OR; // recover and look for the next xml tag
            std::basic_string<wchar_t> str;
            MANY1( 
                if (str.length()>0 && str[str.length()-1]==L'>')
                    PERROR;
                PO(readAnyToken(NULL,&str));
            );
        OPTend;
	}
    ENDBLOCK;

    PARSEend;

    if (p.error()) {
        cout<<"On file "<<path<<": "<<p.errorMessage()<<endl;
        fclose(f);
        exit(1);
    }
    fclose(f);
}

void readResourceFile(const std::string& path,RLoader* r){
    FILE * f=fopen(path.c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open resource file: "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer ct(f);
    ConfigTokenizer t(&ct);
    Parser p=&t;

    PARSEbegin(Parser,p);
        PWS;P1(r->parse,NULL);
    PARSEend;

    if (p.error()) {
        cout<<"On file "<<path<<": "<<p.errorMessage()<<endl;
        fclose(f);
        exit(1);
    }
    fclose(f);
}

==> spacejunk-1.0.5.tar.gz: spacejunk-1.0.5/src/resourcemanager.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourcemanager.h"
#include <SDL_image.h>
#include <algorithm>
#include <functional>
#include "parsercombinators.h"
#include <fstream>
#include <iostream>
#include <SFont.h>
#include <unistd.h>
#include <global.h>
#include "debugmsg.h"
#include "sge.h"


#if defined(__MACOS__) || defined(__MACOSX__)
#include "CoreFoundation/CoreFoundation.h"
#endif

using namespace std;


std::string * applicationPath;
std::string * resourcePath;
std::string * dataPath;

inline void initPaths() {
    if (!resourcePath) {
#if defined(__MACOS__) || defined(__MACOSX__)
		CFBundleRef mainBundle = CFBundleGetMainBundle();
		if (mainBundle) {
			CFURLRef url = CFBundleCopyResourcesDirectoryURL(mainBundle);
			if (url) {
				int bufsize = 128*1024;
				UInt8 buf[bufsize];
				if (CFURLGetFileSystemRepresentation(url,true,buf,bufsize)) {
					resourcePath = new std::string((char*)buf);
					*resourcePath += "/";
					dataPath = new std::string(*resourcePath);
					*dataPath += "data/";
					applicationPath = new std::string(*resourcePath);
					std::string posfix = "Contents/Resources/";
					if (resourcePath->find_last_of(posfix)==resourcePath->length()-1)
						*applicationPath += "../../../";
					else {
						*applicationPath = "";
						*resourcePath = "";
						*dataPath = "data/";
					}
					CFRelease(url);
					return;
				}
				CFRelease(url);
			}
		} 
#endif
		applicationPath = new std::string("");
		resourcePath = new std::string("");
	    dataPath = new std::string("data/");
	}
}

const std::string& GetApplicationPath(){
	initPaths();
	return *applicationPath;
};

const std::string& GetResourcePath(){
	initPaths();
	return *resourcePath;
};


const std::string& GetDataPath(){
	initPaths();
    return *dataPath;
};


// Spanish special characters
basic_string<wchar_t> spchars;


Mix_Music * MusicLoader::create(const std::string & name) {
    string path=music_paths[name];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Music "<<name<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load sound if it is not loaded
    SDL_RWops *rw=SDL_RWFromFile((GetDataPath()+path).c_str(),"r");
    if (!rw) {
        CHERROR<<SDL_GetError()<<ENDL;
        CHERROR<<"ResourceManager error: File "<<path<<" could not be loaded."<<ENDL;
        exit(1);
    }

    Mix_Music * s=Mix_LoadMUS_RW(rw);
    if (!s) {
        CHERROR<<SDL_GetError()<<ENDL;
        CHERROR<<"ResourceManager error: Music file "<<path<<" could not be loaded."<<ENDL;
        exit(1);
    }

    return s;
};

Parser & MusicLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    S("<musics>");
    MANYbegin;
    TRY(S("<music"))WS;
    S("name=")PATTTEXT(&name);
    WS;
    S("path=")PATTTEXT(&path)S("/>")WS;
    music_paths[name]=path;
    MANYend;
    S("</musics>");
    WS;
    PARSEend;
    return p;
};


Mix_Chunk * SoundLoader::create(const std::string & name) {
    string path=sound_paths[name];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Sound "<<name<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load sound if it is not loaded
    SDL_RWops *rw=SDL_RWFromFile((GetDataPath()+path).c_str(),"r");
    if (!rw) {
        CHERROR<<"ResourceManager error: File "<<path<<" could not be loaded. "<<SDL_GetError()<<ENDL;
        exit(1);
    }

    Mix_Chunk * s=Mix_LoadWAV_RW(rw,false);
	SDL_RWclose(rw);
    if (!s) {
        CHERROR<<"ResourceManager error: Sound file "<<ENDL<<" could not be loaded. "<<SDL_GetError()<<ENDL;
        exit(1);
    }

    return s;
};
Parser & SoundLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    LS("<sounds>");
    MANYbegin;
    string text;
    TRY(S("<sound"))WS;
    S("name=")PATTTEXT(&name);
    WS;
    S("path=")PATTTEXT(&path)S("/>")WS;
    sound_paths[name]=path;
    MANYend;
    S("</sounds>");
    WS;
    PARSEend;
    return p;
};



Palette * PaletteLoader::create(const std::string & id) {
    string path=palette_paths[id];
    if (!path.length()) {
        CHERROR<<"ResourceManager error: Palette "<<id<<" is not defined."<<ENDL;
        exit(1);
    }

    // Load palette if it is not loaded
    FILE * f=fopen((GetDataPath()+path).c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open Palette file: "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer tok(f);
    Lexer p(&tok);
    Palette * pal=new Palette();
    int red,green,blue;
    PARSEbegin(Lexer,p);
    pal->ncolors=0;
	NEOLN;PEOLN
	NEOLN;PEOLN
	NEOLN;PEOLN
	NEOLN;PEOLN
    WS;
    MANYbegin;
    if (pal->ncolors==256) break;
    LINT(&red)LINT(&green)LINT(&blue);
    pal->colors[pal->ncolors].r=red;
    pal->colors[pal->ncolors].g=green;
    pal->colors[pal->ncolors].b=blue;
    pal->ncolors++;
    NEOLN;
    WS;
    MANYend;

    PARSEend;
    if (p.error()) {
        fclose(f);
        CHERROR<<"ResourceManager error: Palette file "<<(GetDataPath()+path)<<" could not be loaded: "<<p.errorMessage()<<ENDL;
        exit(1);
    }
    fclose(f);

    return pal;
};
Parser & PaletteLoader::parse(Parser & p,std::string * id) {
    std::string path, name;
    PARSEbegin(Parser,p);
    LS("<palettes>")
    MANYbegin;
    TRY(S("<palette"));
    WS;
    S("name=")PATTTEXT(&name)S("path=")PATTTEXT(&path)S("/>");
    palette_paths[name]=path;
    MANYend;
    S("</palettes>")
    PARSEend;
    return p;
};


void Custom_FreeChunk(Mix_Chunk * mc) {
    for (int i=Mix_AllocateChannels(-1);i>=0;i--)
        if (Mix_GetChunk(i)==&*mc) Mix_HaltChannel(i);
    Mix_FreeChunk(mc);
}

std::string TextLoader::create(const std::string & id) {
    std::map<std::string,std::string>::iterator i=texts.find(id);
    if (i!=texts.end()) 
		return i->second;
    else {
        CHERROR<<"ResourceManager error: Text "<<id<<" is not defined."<<ENDL;
        exit(1);
    }
};
Parser & TextLoader::parse(Parser & p,std::string * id) {
    string name;
    PARSEbegin(Parser,p);
    LS("<texts>");
    MANYbegin;
    basic_string<wchar_t> text;
    TRY(S("<text"))WS;
    S("name=")PATTTEXT(&name)S(">")WS;
    PATTTEXT(&text)
    texts[name]=wstos(text);
    S("</text>")WS;
    MANYend;
    S("</texts>");
    WS;
    PARSEend;
    return p;
};

std::vector<std::string> * LevelLoader::create(const std::string & id) {
    return &levels;
};
Parser & LevelLoader::parse(Parser & p,std::string * id) {
    string name,path;
    PARSEbegin(Parser,p);
    LS("<levels>");
    MANYbegin;
    TRY(S("<level"))WS;
    S("path=")PATTTEXT(&path)S("/>");
    levels.push_back(path);
    MANYend;
    S("</levels>");
    PARSEend;
    return p;
};



void readResources(const std::string& path,std::list<RLoader*> & rl) {
    FILE * f=fopen(path.c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open resource file: "<<path<<": "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer ct(f);
    ConfigTokenizer t(&ct);
    Parser p=&t;

    PARSEbegin(Parser,p);
    PWS;
    S("<resources>")WS;
    S("<specialcharacterset>") PATTTEXT(&spchars)S("</specialcharacterset>")WS;
    for (unsigned int i=0;i<spchars.size();i++)
        SFont_map[spchars[i]-127]=127+i;

    list<RLoader*>::iterator i=rl.begin();
    while(i!=rl.end() && !p.error()) {
        OPTbegin;
            P1((*i)->parse,NULL);
            i++;
        OR; // recover and look for the next xml tag
            std::basic_string<wchar_t> str;
            MANY1( 
                if (str.length()>0 && str[str.length()-1]==L'>')
                    PERROR;
                PO(readAnyToken(NULL,&str));
            );
        OPTend;
	}
    ENDBLOCK;

    PARSEend;

    if (p.error()) {
        cout<<"On file "<<path<<": "<<p.errorMessage()<<endl;
        fclose(f);
        exit(1);
    }
    fclose(f);
}

void readResourceFile(const std::string& path,RLoader* r){
    FILE * f=fopen(path.c_str(),"r");
    if (!f) {
        CHERROR<<"Cannot open resource file: "<<SDL_GetError()<<ENDL;
        exit(1);
    }
    StreamTokenizer ct(f);
    ConfigTokenizer t(&ct);
    Parser p=&t;

    PARSEbegin(Parser,p);
        PWS;P1(r->parse,NULL);
    PARSEend;

    if (p.error()) {
        cout<<"On file "<<path<<": "<<p.errorMessage()<<endl;
        fclose(f);
        exit(1);
    }
    fclose(f);
}
